using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Principal;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using System.Security;

namespace InfiniTec.Security
{
    /// <summary>
    /// Represents a <see cref="Username"/> and <see cref="Password"/> combination.
    /// </summary>
    public class CredentialEntry: IDisposable
    {
        private object _SyncRoot = new object();

        UserToken _Handle;
        string _Username;
        string _Domain;
        SecureString _Password;

        private DateTime _Created;

        /// <summary>
        /// Returns the <see cref="DateTime"/> when the object was created.
        /// </summary>
        public DateTime Created
        {
            get { return _Created; }
        }

        /// <summary>
        /// Gets or sets the username for this <see cref="CredentialEntry"/>
        /// </summary>
        public string Username
        {
            get { _LastAccessed = DateTime.Now; return _Username; }
            set { _Username = value; }
        }
        /// <summary>
        /// Gets or sets the domain for this <see cref="CredentialEntry"/>
        /// </summary>
        public string Domain
        {
            get { _LastAccessed = DateTime.Now; return _Domain; }
            set { _Domain = value; }
        }

        /// <summary>
        /// Gets or sets the password for this <see cref="CredentialEntry"/>
        /// </summary>
        public SecureString Password
        {
            get { _LastAccessed = DateTime.Now; return _Password; }
            set { _Password = value; }
        }

        private DateTime _LastAccessed;
           
        /// <summary>
        /// Returns the <see cref="DateTime"/> when the object was last accessed.
        /// </summary>
        public DateTime LastAccessed
        {
            get { return _LastAccessed; }
        }

        /// <summary>
        /// Returns the <see cref="WindowsIdentity"/> represented by this instance.
        /// </summary>
        public WindowsIdentity Identity
        {
            get { _LastAccessed = DateTime.Now; return GetWindowsIdentity(); }
        }

        /// <summary>
        /// Creates a new instance of this class and logs on as the specified user.
        /// </summary>
        /// <param name="username">The name of the user to log on to (either as 'username' or as 'domain\username')</param>
        /// <param name="password">The password of the useraccount</param>
        [SecurityPermission(SecurityAction.Demand, UnmanagedCode = true)]
        public CredentialEntry(string username, SecureString password)
        {
            string[] parts;

            _Created = DateTime.Now;
            parts = username.Split('\\');
            if (parts.Length == 2)
            {
                _Domain = parts[0];
                _Username = parts[1];
            }
            else _Username = username;

            _Password = password;
        }

        private WindowsIdentity GetWindowsIdentity()
        {
            bool result;
            IntPtr passwordPointer = IntPtr.Zero;

            lock (_SyncRoot)
            {
                try
                {
                    passwordPointer = Marshal.SecureStringToGlobalAllocUnicode(_Password);

                    result = NativeMethods.LogonUser(_Username, _Domain, passwordPointer, (int)NativeMethods.LogonType.LOGON32_LOGON_INTERACTIVE, NativeMethods.LOGON32_PROVIDER_DEFAULT, out _Handle);
                }
                finally
                {
                    if (passwordPointer != IntPtr.Zero) Marshal.ZeroFreeGlobalAllocUnicode(passwordPointer);
                }

                if (!result)
                {
                    Marshal.ThrowExceptionForHR(Marshal.GetHRForLastWin32Error());
                }
            }

            return _Handle.GetWindowsIdentity();
        }

        #region IDisposable Members

        /// <summary>
        /// Releases all resources associated with this instance
        /// </summary>
        [SecurityPermission(SecurityAction.Demand, UnmanagedCode = true)]
        public void Dispose()
        {
            lock (_SyncRoot)
            {
                if (_Handle != null)
                {
                    _Handle.Dispose();
                    _Handle = null;
                }
            }
        }

        #endregion
    }
}
