using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace InfiniTec.Security
{
    /// <summary>
    /// Extends the <see cref="SecurityContext"/> querying the user for a valid 
    /// username/password combination using the <see cref="QueryCredentialDialog"/>.
    /// </summary>
    public class DialogSecurityContext : SecurityContext
    {
        private DialogSecurityContextInfo _Info;
        private QueryCredentialDialog _Dialog;

        /// <summary>
        /// Gets or sets the configuration for this instance.
        /// </summary>
        public DialogSecurityContextInfo Info
        {
            get { return _Info; }
        }

        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        /// <param name="targetName">The targetname and username for this instance</param>
        public DialogSecurityContext(string targetName) : this(new DialogSecurityContextInfo(targetName), true) { }
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        /// <param name="targetName">The targetname and username for this instance</param>
        /// <param name="impersonateImmediately">True, if an impersonation should take place immediately</param>
        public DialogSecurityContext(string targetName, bool impersonateImmediately) : this(new DialogSecurityContextInfo(targetName), impersonateImmediately) { }

        /// <summary>
        /// Initializes a new instance of this class using the specified startup info.
        /// </summary>
        /// <param name="info">The <see cref="DialogSecurityContextInfo"/> for this instance.</param>
        public DialogSecurityContext(DialogSecurityContextInfo info) : this(info, true) { }

        /// <summary>
        /// Creates a new instance of this class and impersonates as the user specified by targetName.
        /// </summary>
        /// <param name="info">The targetName of the identity, as stored in the <see cref="CredentialManager"/>,</param>
        /// <param name="impersonateImmediately">True, if an impersonation should take place immediately</param>
        public DialogSecurityContext(DialogSecurityContextInfo info, bool impersonateImmediately)
            : base(info.TargetName, null, false)
        {
            _Info = info;

            CredentialCallback = AcquireCredential;

            if (impersonateImmediately) Impersonate();
        }

        /// <summary>
        /// Invalidates the credential in the <see cref="CredentialManager"/> as well as
        /// in the Windows Credential manager.
        /// </summary>
        public override void Invalidate()
        {
            base.Invalidate();
            if (_Dialog != null) _Dialog.ConfirmCredentials(false);
        }
        /// <summary>
        /// Confirms the credential in the <see cref="CredentialManager"/> as well as
        /// in the Windows Credential manager.
        /// </summary>
        public override void Confirm()
        {
            base.Confirm();
            if (_Dialog != null) _Dialog.ConfirmCredentials(true);
            
        }
        private void AcquireCredential(object source, AcquireCredentialCallbackArgs e)
        {
            _Dialog = new QueryCredentialDialog();
            _Dialog.CaptionText = Info.Caption;
            _Dialog.MessageText = Info.MessageText;
            _Dialog.Image = Info.Image;
            _Dialog.Username = Info.DefaultUsername;
            _Dialog.Options = Info.Options;

            if (e.AttemptCount != 0)
            {
                _Dialog.Options |= QueryCredentialOptions.IncorrectPassword;
            }
            else
            {
                _Dialog.Options &= ~QueryCredentialOptions.IncorrectPassword;
            }

            _Dialog.TargetName = e.TargetName;
            _Dialog.TargetName = e.TargetName;
            if (_Dialog.ShowDialog() != DialogResult.OK)
            {
                e.Cancel = true;
                return;
            }
            Info.DefaultUsername = _Dialog.Username;
            e.Username = _Dialog.Username;
            e.Password = _Dialog.Password;
            e.Cancel = false;
        }

    }
}