using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Principal;
using System.Security.Permissions;
using System.Security;
using System.Diagnostics;
using System.ComponentModel;
using System.Runtime.InteropServices;

[assembly: SecurityPermission(SecurityAction.RequestMinimum, UnmanagedCode = true)]
namespace InfiniTec.Security
{
    /// <summary>
    /// Encapsulates the impersonation process.
    /// </summary>
    /// <example>
    /// using (SecurityContext context = new SecurityContext(theIdentity, callback)) {
    ///     // do something here
    /// }
    /// </example>
    public class SecurityContext: IDisposable
    {
        private WindowsImpersonationContext _Context;
        private CredentialEntry _Entry;
        private string _TargetName;
        private AcquireCredentialCallback _CredentialCallback;

        /// <summary>
        /// Returns the username of the current impersonation context
        /// </summary>
        public string Username
        {
            get
            {
                EnsureValidCrendentialEntry();
                return _Entry.Username;
            }
        }
        private void EnsureValidCrendentialEntry()
        {
            EnsureValidCrendentialEntry(0);
        }

        private void EnsureValidCrendentialEntry(int attemptCount)
        {
            if (_Entry == null && !AcquireCredential(attemptCount)) HandleInvalidCredential(null);
        }

        /// <summary>
        /// Returns the password of the current impersonation context
        /// </summary>
        public SecureString Password
        {
            get
            {
                EnsureValidCrendentialEntry();
                return _Entry.Password;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="AcquireCredentialCallback"/> method that is called whenever
        /// a valid username/password combination cannot be found in the <see cref="CredentialManager"/>.
        /// </summary>
        protected AcquireCredentialCallback CredentialCallback
        {
            get { return _CredentialCallback; }
            set { _CredentialCallback = value; }
        }

        /// <summary>
        /// Returns true, if the current thread is in impersonation mode. False, otherwise.
        /// </summary>
        public bool Impersonated
        {
            get { return _Context != null; }
        }

        /// <summary>
        /// Impersonates as another user. Calls the specified <see cref="AcquireCredentialCallback"/>
        /// method to get a valid username/password combination, if no credential set can be found
        /// in the <see cref="CredentialManager"/> for the given target name.
        /// </summary>
        public void Impersonate()
        {
            int attemptCount = 1;

            if (Impersonated) throw new InvalidOperationException("Already impersonated!");

            EnsureValidCrendentialEntry();

            do
            {
                try
                {
                    _Context = _Entry.Identity.Impersonate();
                    CacheCredential();
                    break;
                }
                catch (COMException ex)
                {
                    if (ex.ErrorCode != -2147023570) throw;

                    Invalidate();
                    EnsureValidCrendentialEntry(attemptCount);
                }
                attemptCount++;
            } while (true);
        }

        /// <summary>
        /// Creates a new instance of this class and impersonates as the user specified by targetName.
        /// </summary>
        /// <param name="targetName">The targetName of the identity, as stored in the <see cref="CredentialManager"/>,</param>
        /// <param name="callback">This callback is invoked whenever the username and password are no longer available.</param>
        public SecurityContext(string targetName, AcquireCredentialCallback callback) : this(targetName, callback, true) { }

        /// <summary>
        /// Creates a new instance of this class and impersonates as the user specified by targetName.
        /// </summary>
        /// <param name="targetName">The targetName of the identity, as stored in the <see cref="CredentialManager"/>,</param>
        /// <param name="callback">This callback is invoked whenever the username and password are no longer available.</param>
        /// <param name="impersonateImmediately">If true, the impersonation in executed immediately.</param>
        public SecurityContext(string targetName, AcquireCredentialCallback callback, bool impersonateImmediately)
        {
            if (string.IsNullOrEmpty(targetName)) throw new ArgumentException("targetName must not be empty");

            _CredentialCallback = callback;
            _TargetName = targetName;

            if (impersonateImmediately) Impersonate();
        }

        /// <summary>
        /// Starts the given process under the current impersonation context.
        /// </summary>
        /// <param name="info">The process info to use</param>
        public Process StartProcess(ProcessStartInfo info)
        {
            int attemptCount = 1;
            Process result;

            if (Impersonated) throw new InvalidOperationException("Cannot start a process with different credentials when impersonated.");

            EnsureValidCrendentialEntry();

            result = null;

            do
            {
                try
                {
                    info.UserName = _Entry.Username;
                    info.Domain = _Entry.Domain;
                    info.Password = _Entry.Password;
                    result = Process.Start(info);
                    CacheCredential();
                    break;
                }
                catch (Win32Exception ex)
                {
                    if (ex.NativeErrorCode == 1326)
                    {
                        Invalidate();
                        EnsureValidCrendentialEntry(attemptCount);
                    }
                    else throw;
                }
                attemptCount++;
            } while (true);

            return result;
        }

        private void HandleInvalidCredential(Exception ex)
        {
            throw new CredentialInputCancelledException(_TargetName, "No valid credential was provided.", ex);
        }

        /// <summary>
        /// Invalidates the current credential in use. The next time a credential is
        /// needed, it is requeried.
        /// </summary>
        public virtual void Invalidate()
        {
            CredentialManager.Remove(_TargetName);
            _Entry = null;
        }

        /// <summary>
        /// Caches the credential in the credential cache for future use.
        /// </summary>
        public virtual void Confirm()
        {
            CacheCredential();
        }

        private void CacheCredential()
        {
            if (!CredentialManager.Contains(_TargetName)) CredentialManager.Add(_TargetName, _Entry);
        }

        private bool AcquireCredential(int attemptCount)
        {
            AcquireCredentialCallbackArgs e;

            _Entry = CredentialManager.GetIdentity(_TargetName);

            if (_Entry != null) return true;

            if (_CredentialCallback == null) throw new ArgumentException("No callback specified!", "callback");

            e = new AcquireCredentialCallbackArgs(_TargetName, attemptCount);

            _CredentialCallback(this, e);

            if (e.Cancel) return false;

            _Entry = new CredentialEntry(e.Username, e.Password);

            return true;
        }

        #region IDisposable Members

        /// <summary>
        /// Reverts to the original windows identity and releases the resources occupied by this instance
        /// </summary>
        public virtual void Dispose()
        {
            if (!Impersonated)
            {
                RevertToSelf();
            }

        }

        private void RevertToSelf()
        {
            if (_Context != null)
            {
                _Context.Undo();
                _Context.Dispose();
            }
        }

        #endregion
    }
}
